@@ -11,20 +11,20 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 """Module for processing CLI args."""
-import os
+
 import logging
-from awscli.compat import six
+import os
 
 from botocore.compat import OrderedDict, json
-
-from awscli import SCALAR_TYPES, COMPLEX_TYPES
-from awscli import shorthand
-from awscli.utils import (
-    find_service_and_method_in_event_name, is_document_type,
-    is_document_type_container
-)
 from botocore.utils import is_json_value_header
 
+from awscli import COMPLEX_TYPES, SCALAR_TYPES, shorthand
+from awscli.utils import (
+    find_service_and_method_in_event_name,
+    is_document_type,
+    is_document_type_container,
+)
+
 LOG = logging.getLogger('awscli.argprocess')
 
 
@@ -41,9 +41,8 @@ class ParamError(Exception):
         :param message: The error message to display to the user.
 
         """
-        full_message = ("Error parsing parameter '%s': %s" %
-                        (cli_name, message))
-        super(ParamError, self).__init__(full_message)
+        full_message = "Error parsing parameter '%s': %s" % (cli_name, message)
+        super().__init__(full_message)
         self.cli_name = cli_name
         self.message = message
 
@@ -56,16 +55,18 @@ class ParamUnknownKeyError(Exception):
     def __init__(self, key, valid_keys):
         valid_keys = ', '.join(valid_keys)
         full_message = (
-            "Unknown key '%s', valid choices "
-            "are: %s" % (key, valid_keys))
-        super(ParamUnknownKeyError, self).__init__(full_message)
+            f"Unknown key '{key}', valid choices are: {valid_key}"
+        )
+        super().__init__(full_message)
 
 
 class TooComplexError(Exception):
     pass
 
 
-def unpack_argument(session, service_name, operation_name, cli_argument, value):
+def unpack_argument(
+    session, service_name, operation_name, cli_argument, value
+):
     """
     Unpack an argument's value from the commandline. This is part one of a two
     step process in handling commandline arguments. Emits the load-cli-arg
@@ -77,11 +78,12 @@ def unpack_argument(session, service_name, operation_name, cli_argument, value):
     param_name = getattr(cli_argument, 'name', 'anonymous')
 
     value_override = session.emit_first_non_none_response(
-        'load-cli-arg.%s.%s.%s' % (service_name,
-                                   operation_name,
-                                   param_name),
-        param=cli_argument, value=value, service_name=service_name,
-        operation_name=operation_name)
+        f'load-cli-arg.{service_name}.{operation_name}.{param_name}',
+        param=cli_argument,
+        value=value,
+        service_name=service_name,
+        operation_name=operation_name,
+    )
 
     if value_override is not None:
         value = value_override
@@ -103,8 +105,10 @@ def _detect_shape_structure(param, stack):
         if param.type_name in SCALAR_TYPES:
             return 'scalar'
         elif param.type_name == 'structure':
-            sub_types = [_detect_shape_structure(p, stack)
-                        for p in param.members.values()]
+            sub_types = [
+                _detect_shape_structure(p, stack)
+                for p in param.members.values()
+            ]
             # We're distinguishing between structure(scalar)
             # and structure(scalars), because for the case of
             # a single scalar in a structure we can simplify
@@ -142,31 +146,33 @@ def unpack_cli_arg(cli_argument, value):
     :return: The "unpacked" argument than can be sent to the `Operation`
         object in python.
     """
-    return _unpack_cli_arg(cli_argument.argument_model, value,
-                           cli_argument.cli_name)
+    return _unpack_cli_arg(
+        cli_argument.argument_model, value, cli_argument.cli_name
+    )
 
 
 def _special_type(model):
     # check if model is jsonvalue header and that value is serializable
-    if model.serialization.get('jsonvalue') and \
-       model.serialization.get('location') == 'header' and \
-       model.type_name == 'string':
+    if (
+        model.serialization.get('jsonvalue')
+        and model.serialization.get('location') == 'header'
+        and model.type_name == 'string'
+    ):
         return True
     return False
 
 
 def _unpack_cli_arg(argument_model, value, cli_name):
-    if is_json_value_header(argument_model) or \
-            is_document_type(argument_model):
+    if is_json_value_header(argument_model) or is_document_type(
+        argument_model
+    ):
         return _unpack_json_cli_arg(argument_model, value, cli_name)
     elif argument_model.type_name in SCALAR_TYPES:
-        return unpack_scalar_cli_arg(
-            argument_model, value, cli_name)
+        return unpack_scalar_cli_arg(argument_model, value, cli_name)
     elif argument_model.type_name in COMPLEX_TYPES:
-        return _unpack_complex_cli_arg(
-            argument_model, value, cli_name)
+        return _unpack_complex_cli_arg(argument_model, value, cli_name)
     else:
-        return six.text_type(value)
+        return str(value)
 
 
 def _unpack_json_cli_arg(argument_model, value, cli_name):
@@ -174,8 +180,8 @@ def _unpack_json_cli_arg(argument_model, value, cli_name):
         return json.loads(value, object_pairs_hook=OrderedDict)
     except ValueError as e:
         raise ParamError(
-            cli_name, "Invalid JSON: %s\nJSON received: %s"
-            % (e, value))
+            cli_name, f"Invalid JSON: {e}\nJSON received: {value}"
+        )
 
 
 def _unpack_complex_cli_arg(argument_model, value, cli_name):
@@ -183,9 +189,9 @@ def _unpack_complex_cli_arg(argument_model, value, cli_name):
     if type_name == 'structure' or type_name == 'map':
         if value.lstrip()[0] == '{':
             return _unpack_json_cli_arg(argument_model, value, cli_name)
-        raise ParamError(cli_name, "Invalid JSON:\n%s" % value)
+        raise ParamError(cli_name, f"Invalid JSON:\n{value}")
     elif type_name == 'list':
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             if value.lstrip()[0] == '[':
                 return _unpack_json_cli_arg(argument_model, value, cli_name)
         elif isinstance(value, list) and len(value) == 1:
@@ -199,9 +205,10 @@ def _unpack_complex_cli_arg(argument_model, value, cli_name):
             # 2. It's possible this is a list of json objects:
             # --filters '{"Name": ..}' '{"Name": ...}'
             member_shape_model = argument_model.member
-            return [_unpack_cli_arg(member_shape_model, v, cli_name)
-                    for v in value]
-        except (ValueError, TypeError) as e:
+            return [
+                _unpack_cli_arg(member_shape_model, v, cli_name) for v in value
+            ]
+        except (ValueError, TypeError):
             # The list params don't have a name/cli_name attached to them
             # so they will have bad error messages.  We're going to
             # attach the parent parameter to this error message to provide
@@ -212,13 +219,21 @@ def _unpack_complex_cli_arg(argument_model, value, cli_name):
 def unpack_scalar_cli_arg(argument_model, value, cli_name=''):
     # Note the cli_name is used strictly for error reporting.  It's
     # not required to use unpack_scalar_cli_arg
-    if argument_model.type_name == 'integer' or argument_model.type_name == 'long':
+    if (
+        argument_model.type_name == 'integer'
+        or argument_model.type_name == 'long'
+    ):
         return int(value)
-    elif argument_model.type_name == 'float' or argument_model.type_name == 'double':
+    elif (
+        argument_model.type_name == 'float'
+        or argument_model.type_name == 'double'
+    ):
         # TODO: losing precision on double types
         return float(value)
-    elif argument_model.type_name == 'blob' and \
-            argument_model.serialization.get('streaming'):
+    elif (
+        argument_model.type_name == 'blob'
+        and argument_model.serialization.get('streaming')
+    ):
         file_path = os.path.expandvars(value)
         file_path = os.path.expanduser(file_path)
         if not os.path.isfile(file_path):
@@ -226,7 +241,7 @@ def unpack_scalar_cli_arg(argument_model, value, cli_name=''):
             raise ParamError(cli_name, msg)
         return open(file_path, 'rb')
     elif argument_model.type_name == 'boolean':
-        if isinstance(value, six.string_types) and value.lower() == 'false':
+        if isinstance(value, str) and value.lower() == 'false':
             return False
         return bool(value)
     else:
@@ -257,8 +272,7 @@ def _is_complex_shape(model):
     return True
 
 
-class ParamShorthand(object):
-
+class ParamShorthand:
     def _uses_old_list_case(self, service_id, operation_name, argument_name):
         """
         Determines whether a given operation for a service needs to use the
@@ -266,27 +280,24 @@ class ParamShorthand(object):
         a single member.
         """
         cases = {
-            'firehose': {
-                'put-record-batch': ['records']
-            },
+            'firehose': {'put-record-batch': ['records']},
             'workspaces': {
                 'reboot-workspaces': ['reboot-workspace-requests'],
                 'rebuild-workspaces': ['rebuild-workspace-requests'],
-                'terminate-workspaces': ['terminate-workspace-requests']
+                'terminate-workspaces': ['terminate-workspace-requests'],
             },
             'elastic-load-balancing': {
                 'remove-tags': ['tags'],
                 'describe-instance-health': ['instances'],
                 'deregister-instances-from-load-balancer': ['instances'],
-                'register-instances-with-load-balancer': ['instances']
-            }
+                'register-instances-with-load-balancer': ['instances'],
+            },
         }
         cases = cases.get(service_id, {}).get(operation_name, [])
         return argument_name in cases
 
 
 class ParamShorthandParser(ParamShorthand):
-
     def __init__(self):
         self._parser = shorthand.ShorthandParser()
         self._visitor = shorthand.BackCompatVisitor()
@@ -322,18 +333,21 @@ class ParamShorthandParser(ParamShorthand):
         if not self._should_parse_as_shorthand(cli_argument, value):
             return
         else:
-            service_id, operation_name = \
-                find_service_and_method_in_event_name(event_name)
+            service_id, operation_name = find_service_and_method_in_event_name(
+                event_name
+            )
             return self._parse_as_shorthand(
-                cli_argument, value, service_id, operation_name)
+                cli_argument, value, service_id, operation_name
+            )
 
-    def _parse_as_shorthand(self, cli_argument, value, service_id,
-                            operation_name):
+    def _parse_as_shorthand(
+        self, cli_argument, value, service_id, operation_name
+    ):
         try:
-            LOG.debug("Parsing param %s as shorthand",
-                        cli_argument.cli_name)
+            LOG.debug("Parsing param %s as shorthand", cli_argument.cli_name)
             handled_value = self._handle_special_cases(
-                cli_argument, value, service_id, operation_name)
+                cli_argument, value, service_id, operation_name
+            )
             if handled_value is not None:
                 return handled_value
             if isinstance(value, list):
@@ -358,15 +372,20 @@ class ParamShorthandParser(ParamShorthand):
             raise ParamError(cli_argument.cli_name, str(e))
         return parsed
 
-    def _handle_special_cases(self, cli_argument, value, service_id,
-                              operation_name):
+    def _handle_special_cases(
+        self, cli_argument, value, service_id, operation_name
+    ):
         # We need to handle a few special cases that the previous
         # parser handled in order to stay backwards compatible.
         model = cli_argument.argument_model
-        if model.type_name == 'list' and \
-           model.member.type_name == 'structure' and \
-           len(model.member.members) == 1 and \
-           self._uses_old_list_case(service_id, operation_name, cli_argument.name):
+        if (
+            model.type_name == 'list'
+            and model.member.type_name == 'structure'
+            and len(model.member.members) == 1
+            and self._uses_old_list_case(
+                service_id, operation_name, cli_argument.name
+            )
+        ):
             # First special case is handling a list of structures
             # of a single element such as:
             #
@@ -379,11 +398,13 @@ class ParamShorthandParser(ParamShorthand):
             key_name = list(model.member.members.keys())[0]
             new_values = [{key_name: v} for v in value]
             return new_values
-        elif model.type_name == 'structure' and \
-                len(model.members) == 1 and \
-                'Value' in model.members and \
-                model.members['Value'].type_name == 'string' and \
-                '=' not in value:
+        elif (
+            model.type_name == 'structure'
+            and len(model.members) == 1
+            and 'Value' in model.members
+            and model.members['Value'].type_name == 'string'
+            and '=' not in value
+        ):
             # Second special case is where a structure of a single
             # value whose member name is "Value" can be specified
             # as:
@@ -401,10 +422,14 @@ class ParamShorthandParser(ParamShorthand):
             check_val = value[0]
         else:
             check_val = value
-        if isinstance(check_val, six.string_types) and check_val.strip().startswith(
-                ('[', '{')):
-            LOG.debug("Param %s looks like JSON, not considered for "
-                      "param shorthand.", cli_argument.py_name)
+        if isinstance(check_val, str) and check_val.strip().startswith(
+            ('[', '{')
+        ):
+            LOG.debug(
+                "Param %s looks like JSON, not considered for "
+                "param shorthand.",
+                cli_argument.py_name,
+            )
             return False
         model = cli_argument.argument_model
         return _supports_shorthand_syntax(model)
@@ -422,8 +447,9 @@ class ParamShorthandDocGen(ParamShorthand):
             return _supports_shorthand_syntax(argument_model)
         return False
 
-    def generate_shorthand_example(self, cli_argument, service_id,
-                                   operation_name):
+    def generate_shorthand_example(
+        self, cli_argument, service_id, operation_name
+    ):
         """Generate documentation for a CLI argument.
 
         :type cli_argument: awscli.arguments.BaseCLIArgument
@@ -438,7 +464,8 @@ class ParamShorthandDocGen(ParamShorthand):
 
         """
         docstring = self._handle_special_cases(
-            cli_argument, service_id, operation_name)
+            cli_argument, service_id, operation_name
+        )
         if docstring is self._DONT_DOC:
             return None
         elif docstring:
@@ -458,22 +485,27 @@ class ParamShorthandDocGen(ParamShorthand):
 
     def _handle_special_cases(self, cli_argument, service_id, operation_name):
         model = cli_argument.argument_model
-        if model.type_name == 'list' and \
-                model.member.type_name == 'structure' and \
-                len(model.member.members) == 1 and \
-                self._uses_old_list_case(
-                    service_id, operation_name, cli_argument.name):
+        if (
+            model.type_name == 'list'
+            and model.member.type_name == 'structure'
+            and len(model.member.members) == 1
+            and self._uses_old_list_case(
+                service_id, operation_name, cli_argument.name
+            )
+        ):
             member_name = list(model.member.members)[0]
             # Handle special case where the min/max is exactly one.
             metadata = model.metadata
+            cli_name = cli_argument.cli_name
             if metadata.get('min') == 1 and metadata.get('max') == 1:
-                return '%s %s1' % (cli_argument.cli_name, member_name)
-            return '%s %s1 %s2 %s3' % (cli_argument.cli_name, member_name,
-                                       member_name, member_name)
-        elif model.type_name == 'structure' and \
-                len(model.members) == 1 and \
-                'Value' in model.members and \
-                model.members['Value'].type_name == 'string':
+                return f'{cli_name} {member_name}1'
+            return f'{cli_name} {member_name}1 {member_name}2 {member_name}3'
+        elif (
+            model.type_name == 'structure'
+            and len(model.members) == 1
+            and 'Value' in model.members
+            and model.members['Value'].type_name == 'string'
+        ):
             return self._DONT_DOC
         return ''
 
