@@ -20,8 +20,11 @@ from datetime import datetime, timedelta
 from botocore.signers import RequestSigner
 from botocore.model import ServiceId
 
+from awscli.formatter import get_formatter
+from awscli.utils import create_nested_client
 from awscli.customizations.commands import BasicCommand
 from awscli.customizations.utils import uni_print
+from awscli.customizations.utils import validate_mutually_exclusive
 
 AUTH_SERVICE = "sts"
 AUTH_COMMAND = "GetCallerIdentity"
@@ -62,7 +65,7 @@ TOKEN_EXPIRATION_MINS = 14
 
 TOKEN_PREFIX = 'k8s-aws-v1.'
 
-CLUSTER_NAME_HEADER = 'x-k8s-aws-id'
+K8S_AWS_ID_HEADER = 'x-k8s-aws-id'
 
 
 class GetTokenCommand(BasicCommand):
@@ -78,14 +81,26 @@ class GetTokenCommand(BasicCommand):
         {
             'name': 'cluster-name',
             'help_text': (
-                "Specify the name of the Amazon EKS cluster to create a token for."
+                "Specify the name of the Amazon EKS cluster to create a token for. (Note: for local clusters on AWS Outposts, please use --cluster-id parameter)"
             ),
-            'required': True,
+            'required': False,
         },
         {
             'name': 'role-arn',
             'help_text': (
-                "Assume this role for credentials when signing the token."
+                "Assume this role for credentials when signing the token. "
+                "Use this optional parameter when the credentials for signing "
+                "the token differ from that of the current role session. "
+                "Using this parameter results in new role session credentials "
+                "that are used to sign the token."
+            ),
+            'required': False,
+        },
+        {
+            'name': 'cluster-id',
+            # When EKS in-region cluster supports cluster-id, we will need to update this help text
+            'help_text': (
+                "Specify the id of the Amazon EKS cluster to create a token for. (Note: for local clusters on AWS Outposts only)"
             ),
             'required': False,
         },
@@ -102,7 +117,17 @@ class GetTokenCommand(BasicCommand):
         sts_client = client_factory.get_sts_client(
             region_name=parsed_globals.region, role_arn=parsed_args.role_arn
         )
-        token = TokenGenerator(sts_client).get_token(parsed_args.cluster_name)
+        
+        validate_mutually_exclusive(parsed_args, ['cluster_name'], ['cluster_id'])
+
+        if parsed_args.cluster_id:
+            identifier = parsed_args.cluster_id
+        elif parsed_args.cluster_name:
+            identifier = parsed_args.cluster_name
+        else:
+            return ValueError("Either parameter --cluster-name or --cluster-id must be specified.")
+
+        token = TokenGenerator(sts_client).get_token(identifier)
 
         # By default STS signs the url for 15 minutes so we are creating a
         # rfc3339 timestamp with expiration in 14 minutes as part of the token, which
@@ -119,15 +144,21 @@ class GetTokenCommand(BasicCommand):
             },
         }
 
-        uni_print(json.dumps(full_object))
+        output = parsed_globals.output
+        if output is None:
+            output = self._session.get_config_variable('output')
+        formatter = get_formatter(output, parsed_globals)
+        formatter.query = parsed_globals.query
+
+        formatter(self.NAME, full_object)
         uni_print('\n')
         return 0
 
     def discover_api_version(self):
         """
         Parses the KUBERNETES_EXEC_INFO environment variable and returns the
-        API version. If the environment variable is empty, malformed, or
-        invalid, return the v1alpha1 response and print a message to stderr.
+        API version. If the environment variable is malformed or invalid,
+        return the v1beta1 response and print a message to stderr.
 
         If the v1alpha1 API is specified explicitly, a message is printed to
         stderr with instructions to update.
@@ -139,7 +170,7 @@ class GetTokenCommand(BasicCommand):
         # "v1beta1" will be removed. At or around that time, EKS will likely
         # support v1.22 through v1.28, in which client API version "v1beta1"
         # will be supported by all EKS versions.
-        fallback_api_version = ALPHA_API
+        fallback_api_version = BETA_API
 
         error_prefixes = {
             "error": "Error parsing",
@@ -148,16 +179,12 @@ class GetTokenCommand(BasicCommand):
 
         exec_info_raw = os.environ.get("KUBERNETES_EXEC_INFO", "")
         if not exec_info_raw:
-            # All kube clients should be setting this. Otherewise, we'll return
-            # the fallback and write an error.
-            uni_print(
-                ERROR_MSG_TPL.format(
-                    error_prefixes["empty"],
-                    fallback_api_version,
-                ),
-                sys.stderr,
-            )
-            uni_print("\n", sys.stderr)
+            # All kube clients should be setting this, but client-go clients
+            # (kubectl, kubelet, etc) < 1.20 were not setting this if the API
+            # version defined in the kubeconfig was not v1alpha1.
+            #
+            # This was changed in kubernetes/kubernetes#95489 so that
+            # KUBERNETES_EXEC_INFO is always provided
             return fallback_api_version
         try:
             exec_info = json.loads(exec_info_raw)
@@ -177,7 +204,7 @@ class GetTokenCommand(BasicCommand):
         if api_version_raw in FULLY_SUPPORTED_API_VERSIONS:
             return api_version_raw
         elif api_version_raw in DEPRECATED_API_VERSIONS:
-            uni_print(DEPRECATION_MSG_TPL.format(ALPHA_API), sys.stderr)
+            uni_print(DEPRECATION_MSG_TPL.format(api_version_raw), sys.stderr)
             uni_print("\n", sys.stderr)
             return api_version_raw
         else:
@@ -193,18 +220,18 @@ class TokenGenerator(object):
     def __init__(self, sts_client):
         self._sts_client = sts_client
 
-    def get_token(self, cluster_name):
+    def get_token(self, k8s_aws_id):
         """Generate a presigned url token to pass to kubectl."""
-        url = self._get_presigned_url(cluster_name)
+        url = self._get_presigned_url(k8s_aws_id)
         token = TOKEN_PREFIX + base64.urlsafe_b64encode(
             url.encode('utf-8')
         ).decode('utf-8').rstrip('=')
         return token
 
-    def _get_presigned_url(self, cluster_name):
+    def _get_presigned_url(self, k8s_aws_id):
         return self._sts_client.generate_presigned_url(
             'get_caller_identity',
-            Params={'ClusterName': cluster_name},
+            Params={K8S_AWS_ID_HEADER: k8s_aws_id},
             ExpiresIn=URL_TIMEOUT,
             HttpMethod='GET',
         )
@@ -221,32 +248,30 @@ class STSClientFactory(object):
             client_kwargs['aws_access_key_id'] = creds['AccessKeyId']
             client_kwargs['aws_secret_access_key'] = creds['SecretAccessKey']
             client_kwargs['aws_session_token'] = creds['SessionToken']
-        sts = self._session.create_client('sts', **client_kwargs)
-        self._register_cluster_name_handlers(sts)
+        sts = create_nested_client(self._session, 'sts', **client_kwargs)
+        self._register_k8s_aws_id_handlers(sts)
         return sts
 
     def _get_role_credentials(self, region_name, role_arn):
-        sts = self._session.create_client('sts', region_name)
+        sts = create_nested_client(self._session, 'sts', region_name=region_name)
         return sts.assume_role(
             RoleArn=role_arn, RoleSessionName='EKSGetTokenAuth'
         )['Credentials']
 
-    def _register_cluster_name_handlers(self, sts_client):
+    def _register_k8s_aws_id_handlers(self, sts_client):
         sts_client.meta.events.register(
             'provide-client-params.sts.GetCallerIdentity',
-            self._retrieve_cluster_name,
+            self._retrieve_k8s_aws_id,
         )
         sts_client.meta.events.register(
             'before-sign.sts.GetCallerIdentity',
-            self._inject_cluster_name_header,
+            self._inject_k8s_aws_id_header,
         )
 
-    def _retrieve_cluster_name(self, params, context, **kwargs):
-        if 'ClusterName' in params:
-            context['eks_cluster'] = params.pop('ClusterName')
+    def _retrieve_k8s_aws_id(self, params, context, **kwargs):
+        if K8S_AWS_ID_HEADER in params:
+            context[K8S_AWS_ID_HEADER] = params.pop(K8S_AWS_ID_HEADER)
 
-    def _inject_cluster_name_header(self, request, **kwargs):
-        if 'eks_cluster' in request.context:
-            request.headers[CLUSTER_NAME_HEADER] = request.context[
-                'eks_cluster'
-            ]
+    def _inject_k8s_aws_id_header(self, request, **kwargs):
+        if K8S_AWS_ID_HEADER in request.context:
+            request.headers[K8S_AWS_ID_HEADER] = request.context[K8S_AWS_ID_HEADER]
